@@ -1,7 +1,7 @@
 # 2022.7.4 merge verbnet and terms code 
 # 2022.5.28, redisgears's python is 3.7 , to be compatible 
 # 2022.3.20, usage:  import en |  spacy.nlp("hello")   
-import json,spacy,os,builtins
+import json,spacy,os,builtins,time,fileinput, sqlite3,traceback,sys
 from spacy.tokens import DocBin,Doc,Token
 from spacy.tokenizer import Tokenizer
 from spacy.matcher import Matcher,DependencyMatcher
@@ -23,39 +23,7 @@ def custom_tokenizer(nlp): #https://stackoverflow.com/questions/58105967/spacy-t
 		]
 	)
 	infix_re = compile_infix_regex(infixes)
-	return Tokenizer(nlp.vocab, prefix_search=nlp.tokenizer.prefix_search,
-								suffix_search=nlp.tokenizer.suffix_search,
-								infix_finditer=infix_re.finditer,
-								token_match=nlp.tokenizer.token_match,
-								rules=nlp.Defaults.tokenizer_exceptions)
-
-def spacydoc(snt, use_cache=True): 
-	''' added 2022.3.20 '''
-	from en.spacybs import Spacybs
-	if not use_cache: return spacy.nlp(snt)
-	if not hasattr(spacydoc, 'db'): spacydoc.db = Spacybs("spacy311.sqlite")
-	bs = spacydoc.db[snt]
-	if bs is not None : return spacy.frombs(bs)
-	doc = spacy.nlp(snt) 
-	spacydoc.db[snt] = spacy.tobs(doc) 
-	spacydoc.db.conn.commit()
-	return doc 
-
-def sqlitedoc(snt, table): 
-	'''multiple tables supported,  added 2022.3.27 '''
-	import sqlite3
-	if not hasattr(sqlitedoc, 'conn'): 
-		sqlitedoc.conn =	sqlite3.connect("spacy311.sntbs", check_same_thread=False) 
-		sqlitedoc.conn.execute('PRAGMA synchronous=OFF')
-
-	sqlitedoc.conn.execute(f'CREATE TABLE IF NOT EXISTS {table} (key varchar(512) PRIMARY KEY, value blob)')
-	item = sqlitedoc.conn.execute(f'SELECT value FROM "{table}" WHERE key = ? limit 1', (key,)).fetchone()
-	if item	is not None: return spacy.frombs(item[0]) 
-
-	doc = spacy.nlp(snt) 
-	sqlitedoc.conn.execute(f'REPLACE	INTO {table} (key,	value) VALUES (?,?)',	(snt, spacy.tobs(doc) ))
-	sqlitedoc.conn.commit()
-	return doc 
+	return Tokenizer(nlp.vocab, prefix_search=nlp.tokenizer.prefix_search,suffix_search=nlp.tokenizer.suffix_search,infix_finditer=infix_re.finditer,token_match=nlp.tokenizer.token_match,	rules=nlp.Defaults.tokenizer_exceptions)
 
 def to_docbin(doc):
 	doc_bin= spacy.tokens.DocBin()
@@ -73,13 +41,16 @@ if not hasattr(spacy, 'nlp'):
 	spacy.nlp.tokenizer = custom_tokenizer(spacy.nlp)	#nlp.tokenizer.infix_finditer = infix_re.finditer
 	#print([t.text for t in nlp("It's 1.50, up-scaled haven't")]) # ['It', "'s", "'", '1.50', "'", ',', 'up-scaled', 'have', "n't"]
 
-def rgetdoc(rbs, snt, prefix="bs:", ttl:int=37200): 
-	''' bytes , 2022.5.28 '''
-	bs = rbs.get( prefix + snt) 
-	if bs: return Doc(spacy.nlp.vocab).from_bytes(bs)  # spacy.frombs(bs) 
-	doc = spacy.nlp(snt) 
-	rbs.setex(prefix + snt, ttl, doc.to_bytes()) 
-	return doc 
+def refresh(): # to release spacy's memory , added 2023.1.20
+	spacy.nlp = spacy.load(os.getenv('spacy_model','en_core_web_lg'))
+
+def model(name:str='sm'):
+	''' sm/md/lg, doc = en.model('lg')("hello world"),  2023.1.17 '''
+	if not hasattr(model, name):  
+		setattr(model, name, spacy.load(f'en_core_web_{name}') )
+	return getattr(model, name) 
+lookup	= lambda s, name='lg': model(name).vocab[s].text
+fromjson= lambda jsons, name='lg': Doc(model(name).vocab).from_json(jsons)
 
 def getdoc(snt, bs):  # execute("GET", f"bytes:{snt}") 
 	return Doc(spacy.nlp.vocab).from_bytes(bs) if bs else spacy.nlp(snt) 
@@ -128,28 +99,6 @@ def pred_offset(doc):
 	offset = ar[0] if len(ar) > 0 else 0
 	return offset/( len(doc) + 0.1) 
 
-def JSONEachRow(snt): #[{'i': 0, 'head': 1, 'lex': 'The', 'lem': 'the', 'pos': 'DET', 'tag': 'DT', 'dep': 'det', 'gov': 'boy_NOUN', 'chunks': []}, {'i': 1, 'head': 2, 'lex': 'boy', 'lem': 'boy', 'pos': 'NOUN', 'tag': 'NN', 'dep': 'nsubj', 'gov': 'be_AUX', 'chunks': [{'lempos': 'boy_NOUN', 'type': 'NP', 'chunk': 'the boy'}]}, {'i': 2, 'head': 2, 'lex': 'is', 'lem': 'be', 'pos': 'AUX', 'tag': 'VBZ', 'dep': 'ROOT', 'gov': 'be_AUX', 'chunks': []}, {'i': 3, 'head': 2, 'lex': 'happy', 'lem': 'happy', 'pos': 'ADJ', 'tag': 'JJ', 'dep': 'acomp', 'gov': 'be_AUX', 'chunks': []}, {'i': 4, 'head': 2, 'lex': '.', 'lem': '.', 'pos': 'PUNCT', 'tag': '.', 'dep': 'punct', 'gov': 'be_AUX', 'chunks': []}]
-	''' added 2022.6.25 ''' 
-	doc = spacy.nlp(snt) 
-	dic = {}
-	for t in doc:
-		dic[t.i] = {'i':t.i, "head":t.head.i, 'offset':t.idx, 'lex':t.text, 'lem':t.lemma_, 'pos':t.pos_, 'tag':t.tag_, 'dep':t.dep_, "gov":t.head.lemma_ + "_" + t.head.pos_, "chunks":[] }
-	for sp in doc.noun_chunks: 
-		dic[ sp.end - 1 ]['chunks'].append( {'lempos': doc[sp.end - 1].lemma_ + "_NOUN", "type":"NP", "chunk":sp.text.lower() } ) ## start/end ? 
-	return [ v for v in dic.values()]
-
-def parse(snt, merge_np= False):
-	''' used in the notebook, for debug '''
-	import pandas as pd
-	doc = spacy.nlp(snt)
-	if merge_np : spacy.merge_nps(doc)
-	return pd.DataFrame({'word': [t.text for t in doc], 'tag': [t.tag_ for t in doc],'pos': [t.pos_ for t in doc],'head': [t.head.orth_ for t in doc],'dep': [t.dep_ for t in doc], 'lemma': [t.text.lower() if t.lemma_ == '-PRON-' else t.lemma_ for t in doc],
-	'n_lefts': [ t.n_lefts for t in doc], 'left_edge': [ t.left_edge.text for t in doc], 
-	'n_rights': [ t.n_rights for t in doc], 'right_edge': [ t.right_edge.text for t in doc],
-	'subtree': str([ list(t.subtree) for t in doc]),'children': str([ list(t.children) for t in doc]),
-	'morph': [ t.morph for t in doc],
-	}) 
-
 def show(doc):
 	''' used in the notebook, for debug '''
 	import pandas as pd
@@ -160,6 +109,12 @@ def show(doc):
 	'morph': [ t.morph for t in doc],'ent_type': [ t.ent_type_ for t in doc],
 	})
 
+def parse(snt, merge_np= False):
+	''' used in the notebook, for debug '''
+	doc = spacy.nlp(snt)
+	if merge_np : spacy.merge_nps(doc)
+	return show(doc)
+
 trp_rel		= lambda t:  f"{t.dep_}_{t.head.pos_}_{t.pos_}"  # dobj_VERB_NOUN
 trp_reverse = set({"amod_NOUN_ADJ","nsubj_VERB_NOUN"})
 trp_tok		= lambda doc, arr:  [ t for t in doc if [ t.dep_, t.head.pos_, t.pos_, t.head.lemma_, t.lemma_ ] == arr ] # arr is exactly 5 list 
@@ -207,6 +162,26 @@ def kp_matcher(doc): #[('vend', 'consider going', 1, 3), ('vp', 'consider going'
 	[tups.add( kp_span(doc,ibeg,iend, spacy.nlp.vocab[name].text) ) for name, ibeg,iend in kp_matcher.matcher(doc)] 
 	return tups
 
+def ruler(snt, name:str='sm'):
+	''' 2023.1.21 '''
+	if not hasattr(ruler, name):  
+		_nlp = spacy.load(f'en_core_web_{name}')
+		setattr(ruler, name, _nlp )
+		_ruler = _nlp.add_pipe("span_ruler")
+		patterns = [{"label": "PP", "pattern": [{'POS': 'ADP'},{"POS": {"IN": ["DET","NUM","ADJ",'PUNCT','CONJ']}, "OP": "*"},{"POS": {"IN": ["NOUN","PART","PROPN"]}, "OP": "+"}]},
+            {"label": "VP", "pattern": [{'POS': 'VERB'},{"POS": {"IN": ["DET","ADP","ADJ"]}, "OP": "*"},{"POS": 'NOUN'}, {"POS": {"IN": ["ADP","TO"]}, "OP": "*"}]},
+			{"label": "Vend", "pattern": [{"POS": {"IN": ["AUX","VERB"]}},{"POS": {"IN": ["ADV"]}, "OP": "*"}, {"POS": {"IN": ["ADJ","VERB"]}, "OP": "*"},{"POS": {"IN": ["PART","ADP","TO"]}, "OP": "*"},{"POS": 'VERB'}]},
+			{"label": "AP", "pattern": [{"POS": {"IN": ["ADV"]}, "OP": "+"}, {"POS": 'ADJ'}]},
+			{"label": "vtov", "pattern": [{"POS": 'VERB', "TAG": {"NOT_IN": ["VBN"]}}, {"TAG": 'TO'},{"TAG": 'VB'}]},
+			{"label": "vvbg", "pattern": [{"POS": 'VERB', "TAG": {"NOT_IN": ["VBN"]}}, {"TAG": 'VBG'}]},
+			{"label": "vprt", "pattern": [{"POS": 'VERB', "TAG": {"NOT_IN": ["VBN"]}}, {"POS": {"IN": ["PREP", "ADP",'TO']}, "OP": "+"}]},
+			{"label": "vpg", "pattern": [{"POS": 'VERB', "TAG": {"NOT_IN": ["VBN"]}}, {"POS": {"IN": ["PREP", "ADP",'PART']}, "OP": "+"},{"TAG": 'VBG'}]},# insisted on going
+			{"label": "be_vbn_p", "pattern": [{'LEMMA': 'be'},{"TAG": {"IN": ["VBN"]}}, {"POS": {"IN": ["PREP", "ADP",'PART']}}]},# base:VERB:be_vbn_p:be based on   
+			{"label": "be_adj_p", "pattern": [{'LEMMA': 'be'},{"POS": {"IN": ["ADJ"]}}, {"POS": {"IN": ["PREP", "ADP",'PART']}}]},# be angry with
+			]
+		_ruler.add_patterns(patterns)
+	return getattr(ruler, name)(snt)  #print([(span.text, span.label_) for span in  doc.spans["ruler"] ])
+
 # added 2022.7.25
 post_np_rules = { # after np is merged 
 "v_n_vbn": [[{"POS": 'VERB', "TAG": {"NOT_IN": ["VBN"]}},{"POS": {"IN": ["NOUN"]}}, {"TAG": {"IN": ["VBN"]}}]],   # leave the book opened 
@@ -236,43 +211,6 @@ matchers = {  # for name,start,end in matchers['ap'](doc) :
 "vap":  new_matcher([[{'LEMMA': 'be'},{"POS": {"IN": ["ADJ"]}}, {"POS": {"IN": ["PREP", "ADP",'PART']}}]]),   # be angry with
 } #for name, ibeg,iend in matcher(doc) : print(spacy.nlp.vocab[name].text, doc[ibeg:iend].text)
 
-def readline(infile, sepa=None): #for line in fileinput.input(infile):
-	with open(infile, 'r') as fp:
-		while True:
-			line = fp.readline()
-			if not line: break
-			yield line.strip().split(sepa) if sepa else line.strip()
-
-# added 2022.6.30 
-def register_redis(rinfo="172.17.0.1:6379:0", force:bool=False):  #rhost='172.17.0.1', rport=6379, rdb=0
-	import redis
-	try:
-		if not hasattr(redis, 'enr') or force:
-			arr = rinfo.strip().split(':')
-			redis.enr = redis.Redis(host=arr[0], port=int(arr[1]), db=int(arr[2]) if len(arr) >= 2 else 0, decode_responses=True)
-			redis.enbs = redis.Redis(host=arr[0],port=int(arr[1]), db=int(arr[2]) if len(arr) >= 2 else 0, decode_responses=False)
-			print ( redis.enr, redis.enbs, flush=True)
-			return True
-	except Exception as e:
-		print ( "exception : ", e) 
-		return False 
-
-def publish_newsnts( snts , name:str='xsntbytes'): 
-	''' '''
-	import redis
-	if hasattr(redis, "enr") and snts: #rbs.setex(prefix + snt, ttl, doc.to_bytes()) 
-		[redis.enr.xadd(name, {'snt':snt}) for snt in snts if snt] # notify spacy snt parser
-
-def parse_if(snt, prefix:str="bytes:", ttl:int=37200): 
-	''' bytes , 2022.6.30 '''
-	import redis
-	if hasattr(redis, "enbs"):
-		bs = redis.enbs.get( prefix + snt) 
-		if bs: return Doc(spacy.nlp.vocab).from_bytes(bs)  # spacy.frombs(bs) 
-	doc =  spacy.nlp(snt) 
-	if hasattr(redis, "enbs"): redis.enbs.setex(prefix + snt, ttl, doc.to_bytes()) 
-	return doc 
-
 ##
 ## from verbnet.py
 ##
@@ -424,24 +362,7 @@ def vp_matcher(doc): #[('vend', 'consider going', 1, 3), ('vp', 'consider going'
 	return tups
 
 _dep_rules = {
-"svo":[ 
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "subject",
-    "RIGHT_ATTRS": {"DEP": "nsubj"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "object",
-    "RIGHT_ATTRS": {"DEP": "dobj"}
-  }
-], # [(4851363122962674176, [2, 0, 4])]
+"svo":[ {"RIGHT_ID": "v",  "RIGHT_ATTRS": {"POS": "VERB"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"}},{"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object","RIGHT_ATTRS": {"DEP": "dobj"}  }], # [(4851363122962674176, [2, 0, 4])]
 "sva":[ 
   {
     "RIGHT_ID": "v",
@@ -678,43 +599,45 @@ def es_postag(doc):
 	from dic.oneself import oneself 
 	return "_^ " + ' '.join([ f"_{t.pos_}_{t.tag_}" if t.pos_ in ('PROPN','NUM','X','SPACE') else f"{t.text if t.text in ('I') else t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}{oneself.get(t.text.lower(), '')}"  for t in doc]) # uniq by ana #,'PUNCT'
 
+# added 2023.1.16, not tested yet
+cl_root = lambda t: t.dep_ != 'ROOT' and ( (t.pos_ == 'VERB' and t.tag_ != 'VBN')  or (t.pos_ == 'AUX' and t.lemma_ == 'be') )
+def mark_cl(doc):
+    for t in doc: 
+        if cl_root(t) : t.ent_id_ = 'cl_root'
+leave_cl = lambda t: cl_root(t) and len([tr for tr in t.subtree if tr.ent_id_ == 'cl_root']) == 1
+def merge_leave_cl(doc): #doc = spacy.nlp("While I was thrilled that it was ok, I worried that I would fail.")
+	mark_cl(doc)
+	with doc.retokenize() as retokenizer:
+		for v in [t for t in doc if leave_cl(t) ] : # non-root
+			try:
+				children = list(v.subtree)
+				start = children[0].i  	
+				end = children[-1].i 
+				attrs = {"pos": v.pos, "tag": v.tag, "dep": v.dep, "lemma":v.lemma, "ent_type": "S." + v.dep_ } # S.advcl ,  S.conj 
+				if v.dep_ not in ('xcomp') and doc[start].lemma_ not in ('to') and doc[start].tag_ not in ('TO'): # skip to-clause  | He made the choice to give all his money away .
+					retokenizer.merge(doc[start : end+1], attrs=attrs)
+			except Exception as e:
+				print ( "merge_leave_cl ex:", e, v )
+	return doc 
+
+def merge_ccomp(doc): #doc = spacy.nlp("While I was thrilled that it was ok, I worried that I would fail.")
+	with doc.retokenize() as retokenizer:
+		for v in [t for t in doc if t.pos_ == 'VERB' and t.dep_ == 'ccomp' and t.tag_ not in ('VBN') ] : # non-root
+			try:
+				children = list(v.subtree)
+				start = children[0].i  	
+				end = children[-1].i 
+				attrs = {"pos": v.pos, "tag": v.tag, "dep": v.dep, "lemma":v.lemma, "ent_type": "S." + v.dep_ } # S.advcl ,  S.conj 
+				retokenizer.merge(doc[start : end+1], attrs=attrs)
+			except Exception as e:
+				print ( "merge_ccomp ex:", e, v )
+	return doc 
+
 def es_skenp(doc):
 	from dic.oneself import oneself 
-	merge_np(doc) # transform doc , finally to be called 
-	return "_^ " + ' '.join([ f"_{t.lemma_}_NP_{t.dep_}" if t.ent_type_ == 'NP' else f"_{t.pos_}_{t.tag_}_{t.dep_}" if t.pos_ in ('PROPN','NUM','X','SPACE') else f"{t.text if t.text in ('I') else t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}{oneself.get(t.text.lower(), '')}_{t.dep_}"  for t in doc]) # uniq by ana #,'PUNCT'
-
-def kps(doc): 
-	''' doc is of single sent, 2022.9.8 '''
-	_kps = []
-	[ _kps.append(f"{t.pos_}:{t.lemma_}") for t in doc]  # VERB:book
-	[ _kps.append(f"_{t.lemma_}:{t.text.lower()}") for t in doc]  # _book:booked,  added 2022.8.21
-	[ _kps.append(f"{t.tag_}:{t.pos_}_{t.lemma_}") for t in doc]  # VBD:VERB_book,  added 2022.8.25
-	[ _kps.append(f"{t.dep_}:{t.head.pos_}_{t.head.lemma_}:{t.pos_}_{t.lemma_}") for t in doc if t.pos_ not in ('PUNCT')]  # 
-	[ _kps.append(f"~{t.dep_}:{t.pos_}_{t.lemma_}:{t.head.pos_}_{t.head.lemma_}") for t in doc if t.pos_ not in ('PUNCT')]  # 
-	[ _kps.append(f"NP:{doc[sp.end-1].pos_}_{doc[sp.end-1].lemma_}:{sp.text.lower()}") for sp in doc.noun_chunks ]
-	_kps.append( f"stype:" +  "simple_snt" if simple_sent(doc) else "complex_snt" )
-	if compound_snt(doc) : _kps.append("stype:compound_snt")
-
-	# [('pp', 'on the brink', 2, 5), ('ap', 'very happy', 9, 11)]
-	for lem, pos, type, chunk in kp_matcher(doc): #brink:NOUN:pp:on the brink
-		_kps.append(f"{type}:{pos}_{lem}:{chunk}")
-	for trpx, row in dep_matcher(doc): #[('svx', [1, 0, 2])] ## consider:VERB:vnpn:**** 
-		verbi = row[0] #consider:VERB:be_vbn_p:be considered as
-		_kps.append(f"{trpx}:{doc[verbi].pos_}_{doc[verbi].lemma_}")
-		if trpx == 'sva' and doc[row[0]].lemma_ == 'be': # fate is sealed, added 2022.7.25   keep sth. stuck
-			_kps.append(f"sbea:{doc[row[1]].pos_}_{doc[row[1]].lemma_}:{doc[row[2]].pos_}_{doc[row[2]].lemma_}")
-		
-	# last to be called, since NP is merged
-	for row in verbnet_matcher(doc): #[(1, 0, 3, 'NP V S_ING')]
-		if len(row) == 4: 
-			verbi, ibeg, iend, chunk = row
-			if doc[verbi].lemma_.isalpha() : 
-				_kps.append(f"verbnet:{doc[verbi].pos_}_{doc[verbi].lemma_}:{chunk}")
-
-	for name,ibeg,iend in post_np_matcher(doc): #added 2022.7.25
-		if name in ('v_n_vbn','v_n_adj'): 
-			_kps.append(f"{name}:{doc[ibeg].pos_}_{doc[ibeg].lemma_}:{doc[ibeg].lemma_} {doc[ibeg+1].lemma_} {doc[ibeg+2].text}")
-	return _kps
+	merge_ccomp(doc) # merge_leave_cl(doc) 
+	merge_np(doc) # transform doc , finally to be called | f"_{t.lemma_}_{t.pos_}_{t.tag_}_cl_{t.dep_}"
+	return "_^ " + ' '.join([ f"_{t.lemma_}_NP_{t.dep_}" if t.ent_type_ == 'NP' else f"_CL_{t.dep_}" if t.ent_type_.startswith("S.") else f"_{t.pos_}_{t.tag_}_{t.dep_}" if t.pos_ in ('PROPN','NUM','X','SPACE') else f"{t.text if t.text in ('I') else t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}{oneself.get(t.text.lower(), '')}_{t.dep_}"  for t in doc]) # uniq by ana #,'PUNCT'
 
 def merge_prt(doc): 
 	'''I turn off the radio. => turn_off , added 2023.1.13'''
@@ -750,21 +673,23 @@ def depmatch(): #from spacy.matcher import Matcher,DependencyMatcher
 	if not hasattr(depmatch,'matcher'): 
 		depmatch.matcher = DependencyMatcher(spacy.nlp.vocab)
 		pattern = {
-			# be thrilled , worried, 
+			# advcl-acomp worry be thrilled | "While I was thrilled that it was ok, I worried that she is happy."
 			"advcl-acomp": [ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}},  { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "advcl", "RIGHT_ATTRS": {"DEP": "advcl"} }, { "LEFT_ID": "advcl", "REL_OP": ">","RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "acomp"} }] , 
-			"nsubj-acomp": [ { "RIGHT_ID": "v",   "RIGHT_ATTRS": {"POS": "VERB"}},{ "LEFT_ID": "v", "REL_OP":">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"} }, {  "LEFT_ID": "v", "REL_OP": ">",  "RIGHT_ID": "object",    "RIGHT_ATTRS": {"DEP": "acomp"}}], 
-			#She is  a girl.
+			# She is happy. | nsubj-acomp be she happy
+			"nsubj-acomp": [ { "RIGHT_ID": "v",   "RIGHT_ATTRS": {"LEMMA": "be"}},{ "LEFT_ID": "v", "REL_OP":">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"} }, {  "LEFT_ID": "v", "REL_OP": ">",  "RIGHT_ID": "object",    "RIGHT_ATTRS": {"DEP": "acomp"}}], 
+			#She is  a girl. | nsubj-attr be she girl
 			"nsubj-attr": [  {"RIGHT_ID": "v", "RIGHT_ATTRS": {"LEMMA": "be"}}, { "LEFT_ID": "v", "REL_OP": ">","RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"} }, {    "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "attr"} }],
-			# plan to go , enjoy swimming 
+			#  enjoy swimming | nsubj-xcomp enjoy I swimming
 			"nsubj-xcomp": [  {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}},{"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"} }, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object",  "RIGHT_ATTRS": {"DEP": "xcomp"} }],
+			# plan to go , | xcomp-to plan go to
 			"xcomp-to": [  {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object",  "RIGHT_ATTRS": {"DEP": "xcomp"} }, {"LEFT_ID": "object", "REL_OP": ";", "RIGHT_ID": "to", "RIGHT_ATTRS": {"LEMMA": "to"} },],
-			# turn off the light
-			"prt-dobj": [  {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}},  { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "prt"} }, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "dobj"} }],
-			# Some of the Republican policies have left feminists dismayed and aghast.
+			# turn off the light | prt-dobj turn off light
+			"dobj-prt": [  {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}},  { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "prt"} }, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "dobj"} }],
+			# Some of the Republican policies have left feminists dismayed and aghast. | dobj-acl leave feminist dismayed
 			"dobj-acl":[ {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v","REL_OP": ">","RIGHT_ID": "dobj","RIGHT_ATTRS": {"DEP": "dobj"}},{  "LEFT_ID": "dobj", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "acl"}}],
-			# be happy with
+			# be happy with | nsubj-acomp be I happy
 			"acomp-prep":[ {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v","REL_OP": ">","RIGHT_ID": "acomp","RIGHT_ATTRS": {"DEP": "acomp"}},{  "LEFT_ID": "acomp", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "prep"}}],
-			# be based on
+			# be based on | be-vbn-prep base be on
 			"be-vbn-prep":[ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"TAG": "VBN"}}, {"LEFT_ID": "v", "REL_OP": ">","RIGHT_ID": "be","RIGHT_ATTRS": {"LEMMA": "be"} }, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "prep"}  }],
 			}
 		for name,pat in pattern.items(): depmatch.matcher.add(name, [pat])
@@ -772,15 +697,215 @@ def depmatch(): #from spacy.matcher import Matcher,DependencyMatcher
 	#doc = spacy.nlp("I turn off the light, and it is based on the table.")
 	#for name, ar in matcher(doc) : print(spacy.nlp.vocab[name].text, doc[ar[0]].lemma_, doc[ar[1]].lemma_, doc[ar[2]]) # worry be thrilled
 
-[setattr(builtins, k, v) for k, v in globals().items() if not k.startswith("_") and not '.' in k and not hasattr(builtins,k) ] #setattr(builtins, "spacy", spacy)
-if __name__	== '__main__': 
-	doc = spacy.nlp("Some of the Republican policies have left feminists dismayed and aghast.") 
+def trpx(doc, vocab): # spacy.nlp.vocab, 2023.1.21
+	from spacy.matcher import DependencyMatcher
+	if not hasattr(trpx,'matcher'): 
+		trpx.matcher = DependencyMatcher(vocab)
+		pattern = {
+			# advcl-acomp worry be thrilled | "While I was thrilled that it was ok, I worried that she is happy."
+			"advcl-acomp": [ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}},  { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "advcl", "RIGHT_ATTRS": {"DEP": "advcl"} }, { "LEFT_ID": "advcl", "REL_OP": ">","RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "acomp"} }] , 
+			# She is happy. | nsubj-acomp be she happy
+			"nsubj-acomp": [ { "RIGHT_ID": "v",   "RIGHT_ATTRS": {"LEMMA": "be"}},{ "LEFT_ID": "v", "REL_OP":">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"} }, {  "LEFT_ID": "v", "REL_OP": ">",  "RIGHT_ID": "object",    "RIGHT_ATTRS": {"DEP": "acomp"}}], 
+			#She is  a girl. | nsubj-attr be she girl
+			"nsubj-attr": [  {"RIGHT_ID": "v", "RIGHT_ATTRS": {"LEMMA": "be"}}, { "LEFT_ID": "v", "REL_OP": ">","RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"} }, {    "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "attr"} }],
+			#  enjoy swimming | nsubj-xcomp enjoy I swimming
+			"nsubj-xcomp": [  {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}},{"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"} }, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object",  "RIGHT_ATTRS": {"DEP": "xcomp"} }],
+			# plan to go , | xcomp-to plan go to
+			"xcomp-to": [  {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object",  "RIGHT_ATTRS": {"DEP": "xcomp"} }, {"LEFT_ID": "object", "REL_OP": ";", "RIGHT_ID": "to", "RIGHT_ATTRS": {"LEMMA": "to"} },],
+			# turn off the light | prt-dobj turn off light
+			"dobj-prt": [  {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}},  { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "prt"} }, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "dobj"} }],
+			# Some of the Republican policies have left feminists dismayed and aghast. | dobj-acl leave feminist dismayed
+			"dobj-acl":[ {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v","REL_OP": ">","RIGHT_ID": "dobj","RIGHT_ATTRS": {"DEP": "dobj"}},{  "LEFT_ID": "dobj", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "acl"}}],
+			# be happy with | nsubj-acomp be I happy
+			"acomp-prep":[ {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v","REL_OP": ">","RIGHT_ID": "acomp","RIGHT_ATTRS": {"DEP": "acomp"}},{  "LEFT_ID": "acomp", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "prep"}}],
+			# be based on | be-vbn-prep base be on
+			"be-vbn-prep":[ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"TAG": "VBN"}}, {"LEFT_ID": "v", "REL_OP": ">","RIGHT_ID": "be","RIGHT_ATTRS": {"LEMMA": "be"} }, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "prep"}  }],
+			}
+		for name,pat in pattern.items(): trpx.matcher.add(f"trpx:{name}", [pat])
+	for name, ar in trpx.matcher(doc) :
+		doc.spans[vocab[name].text] = [ doc[ar[0]:ar[0]+1], doc[ar[1]:ar[1]+1], doc[ar[2]:ar[2]+1] ]
+	return doc 
+
+def toktrp(doc): # basic for sqlsi 
+	arr = [f"*:sntnum"]
+	for t in doc:
+		[arr.append(s) for s in  ("*:LEX","*:LEM", f"*:{t.pos_}", f"*:{t.tag_}",f"*:{t.dep_}",f"*:~{t.dep_}") ]
+		if t.pos_ in ("VERB","NOUN","ADJ","ADV") : 
+			arr.append( f"{t.tag_}:{t.text.lower()}")  # VBD :  made , added 2022.12.10
+
+		if not t.pos_ in ('PROPN','X', 'PUNCT',"SPACE") and t.is_alpha:
+			arr.extend([f"{t.lemma_}:{t.pos_}",f"{t.lemma_}:LEX:{t.text.lower()}",f"LEM:{t.lemma_.lower()}",f"LEX:{t.text.lower()}",f"{t.pos_}:{t.lemma_.lower()}",f"{t.lemma_.lower()}:{t.pos_}:{t.tag_}",f"*:{t.pos_}:{t.tag_}"]) 
+		if t.pos_ not in ("PROPN","PUNCT","SPACE") and t.is_alpha and t.head.is_alpha:
+			arr.extend([f"{t.head.lemma_}:{t.head.pos_}:{t.dep_}:{t.pos_}:{t.lemma_}", f"{t.head.lemma_}:{t.head.pos_}:{t.dep_}:{t.pos_}",f"{t.head.lemma_}:{t.head.pos_}:{t.dep_}", f"*:{t.head.pos_}:{t.dep_}", f"*:{t.head.pos_}:{t.dep_}:{t.pos_}:{t.head.lemma_}"])
+			if t.dep_ not in ('ROOT'): 
+				arr.extend([f"{t.lemma_}:{t.pos_}:~{t.dep_}:{t.head.pos_}:{t.head.lemma_}", f"{t.lemma_}:{t.pos_}:~{t.dep_}:{t.head.pos_}", f"{t.lemma_}:{t.pos_}:~{t.dep_}", f"*:{t.pos_}:~{t.dep_}", f"*:{t.pos_}:~{t.dep_}:{t.head.pos_}:{t.lemma_}"])
+
+	for name, ar in depmatch()(doc): # trpx
+		t0,t1,t2 = doc[ar[0]],doc[ar[1]],doc[ar[2]]
+		name =  spacy.nlp.vocab[name].text 
+		if name in ( "nsubj-acomp", "nsubj-attr"):  
+			arr.extend([f"{t1.lemma_}:{t1.pos_}:{name}:{t2.pos_}:{t2.lemma_}",f"{t1.lemma_}:{t1.pos_}:{name}:{t2.pos_}",f"{t1.lemma_}:{t1.pos_}:{name}"])
+			arr.extend([f"{t2.lemma_}:{t2.pos_}:~{name}:{t1.pos_}:{t1.lemma_}", f"{t2.lemma_}:{t2.pos_}:~{name}:{t1.pos_}", f"{t2.lemma_}:{t2.pos_}:~{name}"])
+		elif name in ("advcl-acomp","acomp-prep") : 	
+			arr.extend([f"{t0.lemma_}:{t0.pos_}:{name}:{t2.pos_}:{t2.lemma_}",f"{t0.lemma_}:{t0.pos_}:{name}:{t2.pos_}",f"{t0.lemma_}:{t0.pos_}:{name}"])
+			arr.extend([f"{t2.lemma_}:{t2.pos_}:~{name}:{t0.pos_}:{t0.lemma_}",f"{t2.lemma_}:{t2.pos_}:~{name}:{t0.pos_}",f"{t2.lemma_}:{t2.pos_}:~{name}"])
+		elif name in ("dobj-prt") and t2.pos_ not in ('PROPN'): 	
+			arr.append(f"{t0.lemma_}_{t1.lemma_}:{t0.pos_}:dobj:{t2.pos_}:{t2.lemma_}")
+			arr.append(f"{t2.lemma_}:{t2.pos_}:~dobj:{t0.pos_}:{t0.lemma_}_{t1.lemma_}")
+		elif name in ("be-vbn-prep") : 	#base:VERB:be-vbn-prep:be based on
+			arr.extend([f"{t0.lemma_}:{t0.pos_}:{name}:{t1.lemma_} {t0.text.lower()} {t2.text.lower()}",f"{t0.lemma_}:{t0.pos_}:{name}"])
+			
+	for sp in doc.noun_chunks: #book:NOUN:np:a book
+		arr.extend([f"{sp.root.lemma_.lower()}:{sp.root.pos_}:np:{sp.text.lower()}", f"{sp.root.lemma_.lower()}:{sp.root.pos_}:np", f"*:{sp.root.pos_}:np", f"*:np"])
+	for lem, pos, type, chunk in kp_matcher(doc): #brink:NOUN:pp:on the brink  	# [('pp', 'on the brink', 2, 5), ('ap', 'very happy', 9, 11)]
+		arr.extend([f"{lem}:{pos}:{type}:{chunk}", f"{lem}:{pos}:{type}", f"*:{pos}:{type}", f"*:{type}"])
+	for trpx, row in dep_matcher(doc): #[('svx', [1, 0, 2])] ## consider:VERB:vnpn:**** 
+		verbi = row[0] #consider:VERB:be_vbn_p:be considered as
+		arr.extend([f"{doc[verbi].lemma_}:{doc[verbi].pos_}:{trpx}", f"*:{doc[verbi].pos_}:{trpx}", f"*:{trpx}"]) #consider:VERB:svx
+		if trpx == 'sva' and doc[row[0]].lemma_ == 'be': # fate is sealed, added 2022.7.25
+			arr.extend([f"{doc[row[1]].lemma_}:{doc[row[1]].pos_}:sbea:{doc[row[2]].pos_}:{doc[row[2]].lemma_}", f"{doc[row[1]].lemma_}:{doc[row[1]].pos_}:sbea", f"*:{doc[row[1]].pos_}:sbea"])
+			arr.extend([f"{doc[row[2]].lemma_}:{doc[row[2]].pos_}:~sbea:{doc[row[1]].pos_}:{doc[row[1]].lemma_}", f"{doc[row[2]].lemma_}:{doc[row[2]].pos_}:~sbea", f"*:{doc[row[2]].pos_}:~sbea"])
+	for row in verbnet_matcher(doc): #[(1, 0, 3, 'NP V S_ING')] # last to be called, since NP is merged
+		if len(row) == 4: 
+			verbi, ibeg, iend, chunk = row
+			if doc[verbi].lemma_.isalpha() : 
+				arr.append(f"{doc[verbi].lemma_}:{doc[verbi].pos_}:verbnet:{chunk}") #consider:VERB:verbnet:NP V S_ING
+	for name,ibeg,iend in post_np_matcher(doc): #added 2022.7.25
+		if name in ('v_n_vbn','v_n_adj'): 
+			arr.extend([f"{doc[ibeg].lemma_}:{doc[ibeg].pos_}:{name}:{doc[ibeg].lemma_} {doc[ibeg+1].lemma_} {doc[ibeg+2].text}", f"{doc[ibeg].lemma_}:{doc[ibeg].pos_}:{name}", f"*:{doc[ibeg].pos_}:{name}"])
+	return arr 
+
+def sqlconn(outfile, sqls:str=["create table if not exists si( s varchar(64) not null primary key, i int not null default 0) without rowid","create table if not exists st( s varchar(64) not null primary key, t text, cnt int not null default 0) without rowid"]):
+	conn  =	sqlite3.connect(outfile, check_same_thread=False) 
+	for sql in sqls: conn.execute(sql)
+	conn.execute('PRAGMA synchronous=OFF')
+	conn.execute('PRAGMA case_sensitive_like = 1') #conn.execute("PRAGMA cache_size = -512")  		conn.execute('PRAGMA cache_size = 2000000')  # 2 Gb
+	conn.commit()
+	return conn 
+
+def sqlsi(infile, func:str='toktrp', outfile:str=None,batch=1000,): #ibeg:int=0, iend:int=1000000,
+	''' data for mynac, func must exist , func=trpx '''
+	if outfile is None or not outfile: outfile = infile.split('.')[0] + f".sqlsi"
+	print(f"started: [batch={batch}]", infile, outfile , func , flush=True)
+	func = globals()[func] 
+	start = time.time()
+	conn = sqlconn(outfile) 
+	for sid, line in enumerate(fileinput.input(infile,openhook=fileinput.hook_compressed)): 
+		try: #if sid < ibeg : continue |	if sid >= iend: break 
+			arr = json.loads(line.strip()) 
+			doc = spacy.from_json(arr)  
+			for t in doc:  
+				if not t.pos_ in ('PROPN','X', 'PUNCT',"SPACE",'ROOT','NUM') and not t.head.pos_ in ('PROPN','X', 'PUNCT',"SPACE",'ROOT','NUM') and t.is_alpha and t.head.is_alpha:
+					trp = f"{t.head.lemma_}:{t.head.pos_}:{t.dep_}:{t.pos_}:{t.lemma_}"
+					conn.execute(f"INSERT OR IGNORE INTO st(s,t) VALUES(?,?)", (trp,doc.text))
+			for s in func(doc):  
+				conn.execute(f"INSERT INTO si(s,i) VALUES(?,?) ON CONFLICT(s) DO UPDATE SET i = i + 1", (s,1))
+			if (sid) % batch == 0 : 
+				print (f"sid = {sid}, \t| using(s): ", round(time.time() - start,1), flush=True)
+				conn.commit() #conn.close()  #del conn			#conn = sqlconn(outfile) 		#spacy.nlp	= spacy.load('en_core_web_lg')
+				refresh() 
+		except Exception as e:
+			print ("ex:", e, sid, line) 
+			exc_type, exc_value, exc_obj = sys.exc_info() 	
+			traceback.print_tb(exc_obj)
+	conn.commit()
+	conn.execute(f"update st set cnt = (select i from si where si.s = st.s)")
+	conn.commit()
+	print("sqlsi finished:", infile)
+
+def test(): 
+	doc = spacy.nlp("It is based on the book.") # "While I was thrilled that it was ok, I worried that she is happy."
 	for name, ar in depmatch()(doc) : 
 		print(spacy.nlp.vocab[name].text, doc[ar[0]].lemma_, doc[ar[1]].lemma_, doc[ar[2]])
-	print ( es_skenp(doc)) 	
-	#merge_np(doc)
-	#print ( post_np_matcher(doc) ) 
-	#print (parse("I love you."))
-	#print(sntbr("[\u8bd1\u6587]The 55-kilometre Hong Kong Zhuhai-Macau Bridge is an extraordinary engineering. It is the world's longest sea-crossing transportation system combining bridges and tunnels, which joins the three cities of Hong Kong Zhuhai and Macao, cutting the travelling time among them from 3 hours to 30 minutes. The reinforced concrete bridge with huge spans fully not only proves that China has the ability to complete the record-breaking mega-construction, but also will enhance the regional integration and boost the economic growth. It plays a crucial role in the overall plan to develop China’s Great Bay Area, which China intends to turn into one rivaling those of San Francisco, New York and Tokyo in terms of technological innovation and economic prosperity.", with_pid=True))
+	print ( es_skenp(doc)) 
+
+def es_submit(infile, index:str=None, batch=200000, recreate:bool=True, host='172.17.0.1',port=9200): 
+	''' python3 -m api.sntjson-es gzjc.jsonlg.3.4.1.gz'''
+	from elasticsearch import Elasticsearch,helpers
+	import so
+	es	 = Elasticsearch([ f"http://{host}:{port}" ])  
+	if index is None : index = infile.split('.')[0]
+	print(f">>load started: host={host}, index={index} " , infile, index, flush=True )
+	if recreate or not es.indices.exists(index=index): 
+		if es.indices.exists(index=index):es.indices.delete(index=index)
+		es.indices.create(index=index, body=so.config) 
+
+	actions=[]
+	def add(source):  
+		actions.append( {'_op_type':'index', '_index':index, '_id': source['id'], '_source': source } )
+	for sid, line in enumerate(fileinput.input(infile,openhook=fileinput.hook_compressed)): 
+		try:
+			doc = Doc(spacy.nlp.vocab).from_json(json.loads(line.strip())) # add skenp 
+			for t in doc: 	add({"type":"tok", "id": f"{sid}-tok-{t.i}", "sid":sid, 'lex':t.text, 'lem':t.lemma_, 'pos':t.pos_, 'tag':t.tag_, 'dep':t.dep_, "gpos":t.head.pos_, "glem":t.head.lemma_ , "gtag":t.head.tag_ }) #'i':t.i, "head":t.head.i,
+			for sp in doc.noun_chunks: 	add({"type":"np", "id": f"{sid}-NP-{sp.start}", "sid":sid, 'chunk':sp.text, 'lem':doc[sp.end-1].lemma_ })
+			for lem, pos, type, chunk in kp_matcher(doc): #brink:NOUN:pp:on the brink # [('pp', 'on the brink', 2, 5), ('ap', 'very happy', 9, 11)]
+				add({"type":type, "id": f"{sid}-{type}-{chunk}", "sid":sid,  'chunk':chunk, 'lem':lem , "pos":pos}) #"src": doc.text,
+			for name, ar in depmatch()(doc) : 
+				type = spacy.nlp.vocab[name].text # worry be thrilled
+				lem = doc[ar[0]].lemma_
+				add({"type":type, "id": f"{sid}-{type}-{lem}", "sid":sid,  'lem':lem,  'tag':doc[ar[0]].tag_, 'lem1':doc[ar[1]].lemma_, 'lem2':doc[ar[2]].lemma_ , 'tag1':doc[ar[1]].tag_ , 'tag2':doc[ar[2]].tag_ }) 
+			# merged NP must be finally called 
+			add( {'type':'snt',  "id":sid,  'snt':doc.text, 'postag': es_postag(doc), 'tc': len(doc), 'skenp': es_skenp(doc) }  ) #, 
+			if len(actions) >= batch: 
+				helpers.bulk(client=es,actions=actions, raise_on_error=False)
+				print ( sid, actions[-1], flush=True)
+				actions = []
+				refresh() # 2022.1.20 
+		except Exception as e:
+			print("ex:", e, sid)	
+			exc_type, exc_value, exc_obj = sys.exc_info() 	
+			traceback.print_tb(exc_obj)
+	if actions : helpers.bulk(client=es,actions=actions, raise_on_error=False)
+	print(">>load finished:" , infile, index )
+
+[setattr(builtins, k, v) for k, v in globals().items() if not k.startswith("_") and not '.' in k and not hasattr(builtins,k) ] #setattr(builtins, "spacy", spacy)
+if __name__	== '__main__': 
+	#es_submit("gzjc.jsonlg.3.4.1.gz", host='hw13.jukuu.com')
+	import fire
+	fire.Fire({"sqlsi":sqlsi, "test":test, "es_submit":es_submit })
+
+def kps(doc): 
+	''' doc is of single sent, 2022.9.8 '''
+	_kps = []
+	[ _kps.append(f"{t.pos_}:{t.lemma_}") for t in doc]  # VERB:book
+	[ _kps.append(f"_{t.lemma_}:{t.text.lower()}") for t in doc]  # _book:booked,  added 2022.8.21
+	[ _kps.append(f"{t.tag_}:{t.pos_}_{t.lemma_}") for t in doc]  # VBD:VERB_book,  added 2022.8.25
+	[ _kps.append(f"{t.dep_}:{t.head.pos_}_{t.head.lemma_}:{t.pos_}_{t.lemma_}") for t in doc if t.pos_ not in ('PUNCT')]  # 
+	[ _kps.append(f"~{t.dep_}:{t.pos_}_{t.lemma_}:{t.head.pos_}_{t.head.lemma_}") for t in doc if t.pos_ not in ('PUNCT')]  # 
+	[ _kps.append(f"NP:{doc[sp.end-1].pos_}_{doc[sp.end-1].lemma_}:{sp.text.lower()}") for sp in doc.noun_chunks ]
+	_kps.append( f"stype:" +  "simple_snt" if simple_sent(doc) else "complex_snt" )
+	if compound_snt(doc) : _kps.append("stype:compound_snt")
+
+	# [('pp', 'on the brink', 2, 5), ('ap', 'very happy', 9, 11)]
+	for lem, pos, type, chunk in kp_matcher(doc): #brink:NOUN:pp:on the brink
+		_kps.append(f"{type}:{pos}_{lem}:{chunk}")
+	for trpx, row in dep_matcher(doc): #[('svx', [1, 0, 2])] ## consider:VERB:vnpn:**** 
+		verbi = row[0] #consider:VERB:be_vbn_p:be considered as
+		_kps.append(f"{trpx}:{doc[verbi].pos_}_{doc[verbi].lemma_}")
+		if trpx == 'sva' and doc[row[0]].lemma_ == 'be': # fate is sealed, added 2022.7.25   keep sth. stuck
+			_kps.append(f"sbea:{doc[row[1]].pos_}_{doc[row[1]].lemma_}:{doc[row[2]].pos_}_{doc[row[2]].lemma_}")
+		
+	# last to be called, since NP is merged
+	for row in verbnet_matcher(doc): #[(1, 0, 3, 'NP V S_ING')]
+		if len(row) == 4: 
+			verbi, ibeg, iend, chunk = row
+			if doc[verbi].lemma_.isalpha() : 
+				_kps.append(f"verbnet:{doc[verbi].pos_}_{doc[verbi].lemma_}:{chunk}")
+
+	for name,ibeg,iend in post_np_matcher(doc): #added 2022.7.25
+		if name in ('v_n_vbn','v_n_adj'): 
+			_kps.append(f"{name}:{doc[ibeg].pos_}_{doc[ibeg].lemma_}:{doc[ibeg].lemma_} {doc[ibeg+1].lemma_} {doc[ibeg+2].text}")
+	return _kps
+
+#print(sntbr("[\u8bd1\u6587]The 55-kilometre Hong Kong Zhuhai-Macau Bridge is an extraordinary engineering. It is the world's longest sea-crossing transportation system combining bridges and tunnels, which joins the three cities of Hong Kong Zhuhai and Macao, cutting the travelling time among them from 3 hours to 30 minutes. The reinforced concrete bridge with huge spans fully not only proves that China has the ability to complete the record-breaking mega-construction, but also will enhance the regional integration and boost the economic growth. It plays a crucial role in the overall plan to develop China’s Great Bay Area, which China intends to turn into one rivaling those of San Francisco, New York and Tokyo in terms of technological innovation and economic prosperity.", with_pid=True))
 #c:\users\zhang\appdata\local\programs\python\python38\lib\site-packages  
-#/home/ubuntu/.local/lib/python3.8/site-packages/en
\ No newline at end of file
+#/home/ubuntu/.local/lib/python3.8/site-packages/en
+
+'''
+>>> model('lg').vocab
+<spacy.vocab.Vocab object at 0x0000013459154A60>
+>>> len(model('lg').vocab)
+768
+>>> len(model('sm').vocab)
+767
+'''
\ No newline at end of file
