@@ -15,8 +15,9 @@ def run(infile):
 			try:
 				arr = json.loads(line.strip()) 
 				doc = spacy.from_json(arr) 
+				doc = en.merge_prt(doc)   # added 2022.1.13,  turn_off the radio 
 				for t in doc:
-					if not t.lemma_.replace('-','').isalpha() or not t.head.lemma_.replace('-','').isalpha(): continue # added 2022.1.6 
+					if not t.lemma_.replace('-','').replace('_','').isalpha() or not t.head.lemma_.replace('-','').replace('_','').isalpha(): continue # added 2022.1.6 
 					if t.pos_ not in ('SP','PUNCT','PROPN') and t.tag_ not in ('NUM','CD') and not ':' in t.text :
 						trp = f"{t.dep_}:{t.head.pos_}:{t.pos_}:{t.head.tag_}:{t.tag_}:{t.head.lemma_}:{t.lemma_}"  
 						fw.write(f"{trp}\n")
