@@ -8,7 +8,7 @@ import warnings
 warnings.filterwarnings("ignore")
 
 if not hasattr(requests, 'eshost'):
-	requests.eshost	= os.getenv("eshost", "es.corpusly.com:9200") 
+	requests.eshost	= os.getenv("eshost", "172.17.0.1:9200") 
 	requests.es		= Elasticsearch([ f"http://{requests.eshost}" ])  
 
 config = {  
@@ -130,9 +130,10 @@ config = {
 			"ctag": { "type": "keyword"}, # chunk tag
 			"stag": { "type": "keyword"}, # snt tag
 			"term": { "type": "keyword"},
-			"gpos": { "type": "keyword"},
-			"glem": { "type": "keyword"},
-
+			"gpos": { "type": "keyword"},  "glem": { "type": "keyword"}, "gtag": { "type": "keyword"},
+			"lem1": { "type": "keyword"},  "lem2": { "type": "keyword"}, "tag1": { "type": "keyword"}, "tag2": { "type": "keyword"},
+			"VERB": { "type": "keyword"},  "NOUN": { "type": "keyword"}, "ADV": { "type": "keyword"},  "ADJ": { "type": "keyword"},
+			"advcl": { "type": "keyword"},  "acomp": { "type": "keyword"}, "dobj": { "type": "keyword"},  "nsubj": { "type": "keyword"},"xcomp": { "type": "keyword"},  "amod": { "type": "keyword"},  "advmod": { "type": "keyword"},
 			"frame": { "type": "keyword"},
 			"stype": { "type": "keyword"},
 			"page": { "type": "keyword"},
